<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Block\Frontend\LayoutProcessors;

/**
 * {@inheritdoc}
 */
class HirepurchaseProcessor implements \Magento\Checkout\Block\Checkout\LayoutProcessorInterface
{
    /**
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Montonio\Hirepurchase\Gateway\Hirepurchase\Config\Config $config
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Montonio\Hirepurchase\Gateway\Hirepurchase\Config\Config $config
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->config = $config;
    }

    /**
     * {@inheritdoc}
     */
    public function process($jsLayout)
    {

        $enabled = $this->config->isActive();
        if (!$enabled) {
            return $jsLayout;
        }

        $methodCode = $this->config->getCode();

        $layoutPath = $jsLayout['components']['checkout']
            ['children']['steps']
            ['children']['billing-step']
            ['children']['payment']
            ['children']['renders']
            ['children'] ?? false;

        if ($layoutPath) {

            $paymentMethod = [
                'component' => 'Montonio_Hirepurchase/js/view/payment/' . $methodCode,
                'cnf' => [
                    'methodCode' => $methodCode,
                    'title' => false,
                    'serviceDescription' => $this->config->getServiceDescription(),
                    'serviceLogoPath' => $this->config->getServiceLogoPath()
                ],
                'methods' => [
                    $methodCode => [
                        'isBillingAddressRequired' => true
                    ]
                ]

            ];

            $jsLayout['components']['checkout']
                ['children']['steps']
                ['children']['billing-step']
                ['children']['payment']
                ['children']['renders']
                ['children']['montonio'] = $paymentMethod;
        }

        return $jsLayout;
    }
}
