<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler;

use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Quote\Api\CartRepositoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Class responsible for restoring cart session.
 */
class RestoreLastCartSession
{
    /**
     * @var CheckoutSession
     */
    private $checkoutSession;

    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param CheckoutSession $checkoutSession
     * @param CartRepositoryInterface $cartRepository
     * @param LoggerInterface $logger
     */
    public function __construct(
        CheckoutSession $checkoutSession,
        CartRepositoryInterface $cartRepository,
        LoggerInterface $logger
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->cartRepository = $cartRepository;
        $this->logger = $logger;
    }

    /**
     * Execute cart restore action.
     *
     * @return void
     */
    public function execute(): void
    {
        $order = $this->checkoutSession->getLastRealOrder();

        if ($order === null || !$order->getQuoteId()) {
            return;
        }

        try {
            $quote = $this->cartRepository->get($order->getQuoteId());
            $quote->setIsActive(true)->setReservedOrderId(null);
            $this->cartRepository->save($quote);

            $this->checkoutSession->unsLastQuoteId()->unsLastSuccessQuoteId()->unsLastOrderId()->unsLastRealOrderId();
            $this->checkoutSession->replaceQuote($quote);
        } catch (\Throwable $exception) {
            $this->logger->error('Critical error restoring checkout cart.', ['exception' => $exception->getMessage()]);
        }
    }
}
