<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Gateway\Payments\ResponseHandler;

use Magento\Framework\Exception\InputException;
use Montonio\Hirepurchase\Gateway\Payments\Config\Config as PaymentsConfig;

/**
 * Class responsible for parsing payment token retrieved from gateway.
 */
class TokenParser
{
    /**
     * @var PaymentsConfig
     */
    private $paymentsConfig;

    /**
     * @param PaymentsConfig $paymentsConfig
     */
    public function __construct(PaymentsConfig $paymentsConfig)
    {
        $this->paymentsConfig = $paymentsConfig;
    }

    /**
     * Parse payment token.
     *
     * @param string $paymentToken
     * @return array
     * @throws InputException
     */
    public function parseToken(string $paymentToken): array
    {
        if (empty($paymentToken)) {
            throw new InputException(__('Payment token empty.'));
        }

        $environment = $this->paymentsConfig->getValue('environment');
        $privateKey = $this->paymentsConfig->getValue($environment . '_private_key');
        $publicKey = $this->paymentsConfig->getValue($environment . '_public_key');

        $data = (array)$this->paymentsConfig->decodeSignature($paymentToken, $privateKey);

        if (false === $this->paymentsConfig->validateSignedData($data, $publicKey)) {
            throw new InputException(__('Payment token is invalid.'));
        }

        return $data;
    }
}
