<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
declare(strict_types=1);

namespace Montonio\Hirepurchase\Plugin\Controller\Onepage;

/**
 * Handle around plugin on success page
 */
class Success
{
    /**
     *
     * @param \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Montonio\Hirepurchase\Model\Payment\ConfigProvider $configProvider
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory,
        \Magento\Framework\UrlInterface $urlInterface,
        \Montonio\Hirepurchase\Model\Payment\ConfigProvider $configProvider,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->resultRedirectFactory = $resultRedirectFactory;
        $this->urlInterface = $urlInterface;
        $this->paymentsConfigProvider = $configProvider;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     *
     * @param \Magento\Checkout\Controller\Onepage\Success $subject
     * @param callable $proceed
     * @return callable | resultRedirect
     */
    public function aroundExecute(\Magento\Checkout\Controller\Onepage\Success $subject, callable $proceed)
    {

        $methodCode = $this->paymentsConfigProvider::CODE;

        $enabled = $this->paymentsConfigProvider->getConfigObject()->isActive();

        if (!$enabled) {
            return $proceed();
        }

        $default = true;

        //idea here is to get the data from existing objects to reduce load and db usage
        //if we get our data then we can redirect to payment gateway instead of success page
        $lastRealOrder = $subject
                ->getOnePage()
                ->getCheckout()
                ->getLastRealOrder() ?? false;

        $methodInstance = false;

        if (is_object($lastRealOrder->getPayment())) {
            $methodInstance = $lastRealOrder
            ->getPayment()
            ->getMethodInstance() ?? false;
        }

        if (is_object($lastRealOrder) && is_object($methodInstance)) {

            $methodTarget = $this->paymentsConfigProvider::CODE;
            $methodCode = $methodInstance->getCode() ?? false;

            if ($methodCode && $methodCode === $methodTarget) {

                $token = $methodInstance
                            ->getInfoInstance()
                            ->getAdditionalInformation('access_token') ?? false;

                //we might have a response already , no need to redirect then
                $gatewayResponse = $methodInstance
                            ->getInfoInstance()
                            ->getAdditionalInformation('gateway_response') ?? false;

                if ($token && !$gatewayResponse) {

                    $storeId = $lastRealOrder->getStoreId();
                    $environment = $methodInstance
                            ->getConfigData('environment', $storeId);
                    $url = $methodInstance
                            ->getConfigData($environment.'_application_url', $storeId);
                    $url = $url . "/?access_token=" . $token;
                    $result = $this->resultRedirectFactory
                        ->create()->setUrl($url);
                    //in all other cases we fall back to default action
                    $default = false;
                }
            }
        }

        if ($default) {
            $result = $proceed();
        }

        return $result;
    }
}
